"""Configuration for LLM clients."""

from typing import Literal, Optional

from pydantic import BaseModel, Field


class Config(BaseModel):
    """Configuration for LLM client."""

    provider: Literal[
        "openai",
        "anthropic",
        "google",
        "azure",
        "groq",
        "mistral",
        "openrouter",
        "bedrock",
        "xai",
        "cerebras",
        "cohere",
        "perplexity",
        "deepseek",
        "together",
    ] = "openai"
    model: str = "gpt-4"
    api_key: Optional[str] = None
    temperature: float = Field(default=0.7, ge=0.0, le=2.0)
    max_tokens: Optional[int] = Field(default=None, ge=1)
    timeout: int = Field(default=30, ge=1)
    max_retries: int = Field(default=3, ge=0)
    base_url: Optional[str] = None

    class Config:
        """Pydantic config."""

        frozen = True



"""Amazon Bedrock provider implementation."""

from typing import AsyncIterator, Iterator, Optional
import json

try:
    import boto3
    from botocore.config import Config as BotoConfig
except ImportError:
    raise ImportError("boto3 is required for Bedrock. Install with: pip install boto3")

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class BedrockProvider(Provider):
    """Amazon Bedrock provider implementation."""

    def __init__(self, config: Config):
        """Initialize Bedrock provider.
        
        Note: Bedrock uses AWS credentials from environment or ~/.aws/credentials
        api_key can be used to pass AWS region (default: us-east-1)
        """
        self.config = config
        self.region = config.api_key or "us-east-1"
        
        boto_config = BotoConfig(
            read_timeout=config.timeout,
            retries={"max_attempts": config.max_retries},
        )
        
        self.client = boto3.client(
            "bedrock-runtime",
            region_name=self.region,
            config=boto_config,
        )

    def _convert_messages(self, messages: list[Message]) -> tuple[str, list[dict]]:
        """Convert internal messages to Bedrock format.
        
        Returns:
            Tuple of (system_prompt, messages_list)
        """
        system_prompt = ""
        converted = []
        
        for msg in messages:
            if msg.role == "system":
                system_prompt = msg.content
            else:
                converted.append({
                    "role": msg.role,
                    "content": [{"text": msg.content}]
                })
        
        return system_prompt, converted

    def _build_request_body(
        self,
        messages: list[Message],
        model: str,
        temperature: float,
        max_tokens: Optional[int],
    ) -> dict:
        """Build request body for Bedrock."""
        system_prompt, converted_messages = self._convert_messages(messages)
        
        body = {
            "messages": converted_messages,
            "inferenceConfig": {
                "temperature": temperature,
            }
        }
        
        if max_tokens:
            body["inferenceConfig"]["maxTokens"] = max_tokens
        
        if system_prompt:
            body["system"] = [{"text": system_prompt}]
        
        return body

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        body = self._build_request_body(messages, model, temperature, max_tokens)
        
        response = self.client.converse(
            modelId=model,
            **body,
        )
        
        output = response["output"]["message"]
        content = output["content"][0]["text"]
        
        usage = response.get("usage", {})
        usage_dict = {
            "prompt_tokens": usage.get("inputTokens", 0),
            "completion_tokens": usage.get("outputTokens", 0),
            "total_tokens": usage.get("totalTokens", 0),
        }
        
        return Response(
            content=content,
            model=model,
            usage=usage_dict,
            finish_reason=response.get("stopReason", "stop"),
            metadata={"request_id": response["ResponseMetadata"]["RequestId"]},
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        body = self._build_request_body(messages, model, temperature, max_tokens)
        
        response = self.client.converse_stream(
            modelId=model,
            **body,
        )
        
        stream = response.get("stream")
        if stream:
            for event in stream:
                if "contentBlockDelta" in event:
                    delta = event["contentBlockDelta"]["delta"]
                    if "text" in delta:
                        yield StreamChunk(
                            content=delta["text"],
                            finish_reason=None,
                            metadata={},
                        )
                elif "messageStop" in event:
                    stop_reason = event["messageStop"].get("stopReason", "stop")
                    yield StreamChunk(
                        content="",
                        finish_reason=stop_reason,
                        metadata={},
                    )

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion.
        
        Note: Bedrock doesn't have native async support, this uses sync client.
        For true async, use aioboto3 library.
        """
        # For now, use sync client (true async would require aioboto3)
        return self.complete(messages, model, temperature, max_tokens, **kwargs)

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion.
        
        Note: Bedrock doesn't have native async support, this uses sync client.
        For true async, use aioboto3 library.
        """
        # For now, use sync client (true async would require aioboto3)
        for chunk in self.stream(messages, model, temperature, max_tokens, **kwargs):
            yield chunk



"""Tests for configuration."""

import pytest
from pig_llm.config import Config


def test_config_defaults():
    """Test default config values."""
    config = Config()
    assert config.provider == "openai"
    assert config.model == "gpt-4"
    assert config.temperature == 0.7
    assert config.timeout == 30
    assert config.max_retries == 3


def test_config_custom_values():
    """Test custom config values."""
    config = Config(
        provider="anthropic",
        model="claude-3",
        temperature=0.5,
        max_tokens=1000,
    )
    assert config.provider == "anthropic"
    assert config.model == "claude-3"
    assert config.temperature == 0.5
    assert config.max_tokens == 1000


def test_config_temperature_validation():
    """Test temperature validation."""
    with pytest.raises(ValueError):
        Config(temperature=-0.1)
    
    with pytest.raises(ValueError):
        Config(temperature=2.1)


def test_config_max_tokens_validation():
    """Test max tokens validation."""
    with pytest.raises(ValueError):
        Config(max_tokens=0)


def test_config_frozen():
    """Test config is frozen (immutable)."""
    config = Config()
    with pytest.raises(Exception):  # Pydantic ValidationError
        config.temperature = 0.5



"""OpenRouter provider (aggregates multiple providers)."""

from typing import AsyncIterator, Iterator, Optional

import openai

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class OpenRouterProvider(Provider):
    """OpenRouter provider (access multiple models via one API)."""

    def __init__(self, config: Config):
        """Initialize OpenRouter provider.
        
        OpenRouter uses OpenAI-compatible API.
        """
        self.config = config
        
        # OpenRouter uses OpenAI client with custom base URL
        self.client = openai.OpenAI(
            api_key=config.api_key,
            base_url="https://openrouter.ai/api/v1",
            timeout=config.timeout,
            max_retries=config.max_retries,
        )
        
        self.async_client = openai.AsyncOpenAI(
            api_key=config.api_key,
            base_url="https://openrouter.ai/api/v1",
            timeout=config.timeout,
            max_retries=config.max_retries,
        )

    def _convert_messages(self, messages: list[Message]) -> list[dict]:
        """Convert internal messages to OpenRouter/OpenAI format."""
        result = []
        for msg in messages:
            if msg.role == "assistant" and msg.metadata and "tool_calls" in msg.metadata:
                result.append({
                    "role": "assistant",
                    "content": msg.content or None,
                    "tool_calls": msg.metadata["tool_calls"],
                })
            elif msg.role == "tool" and msg.metadata:
                result.append({
                    "role": "tool",
                    "content": msg.content,
                    "tool_call_id": msg.metadata.get("tool_call_id", ""),
                })
            else:
                result.append({"role": msg.role, "content": msg.content})
        return result

    @staticmethod
    def _extract_tool_calls(message) -> list[dict] | None:
        """Extract tool_calls from OpenAI response message."""
        if not hasattr(message, "tool_calls") or not message.tool_calls:
            return None
        return [
            {
                "id": tc.id,
                "type": "function",
                "function": {
                    "name": tc.function.name,
                    "arguments": tc.function.arguments,
                },
            }
            for tc in message.tool_calls
        ]

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        response = self.client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            **kwargs,
        )

        choice = response.choices[0]
        usage = {
            "prompt_tokens": response.usage.prompt_tokens if response.usage else 0,
            "completion_tokens": response.usage.completion_tokens if response.usage else 0,
            "total_tokens": response.usage.total_tokens if response.usage else 0,
        }

        return Response(
            content=choice.message.content or "",
            model=response.model,
            usage=usage,
            finish_reason=choice.finish_reason,
            tool_calls=self._extract_tool_calls(choice.message),
            metadata={"id": response.id},
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        stream = self.client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            stream=True,
            **kwargs,
        )

        for chunk in stream:
            choice = chunk.choices[0]
            if choice.delta.content:
                yield StreamChunk(
                    content=choice.delta.content,
                    finish_reason=choice.finish_reason,
                    metadata={"id": chunk.id},
                )

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        response = await self.async_client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            **kwargs,
        )

        choice = response.choices[0]
        usage = {
            "prompt_tokens": response.usage.prompt_tokens if response.usage else 0,
            "completion_tokens": response.usage.completion_tokens if response.usage else 0,
            "total_tokens": response.usage.total_tokens if response.usage else 0,
        }

        return Response(
            content=choice.message.content or "",
            model=response.model,
            usage=usage,
            finish_reason=choice.finish_reason,
            tool_calls=self._extract_tool_calls(choice.message),
            metadata={"id": response.id},
        )

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        stream = await self.async_client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            stream=True,
            **kwargs,
        )

        async for chunk in stream:
            choice = chunk.choices[0]
            if choice.delta.content:
                yield StreamChunk(
                    content=choice.delta.content,
                    finish_reason=choice.finish_reason,
                    metadata={"id": chunk.id},
                )



"""Main LLM client."""

from typing import Iterator, Optional

from .config import Config
from .models import Message, Response, StreamChunk


class LLM:
    """Unified LLM client supporting multiple providers."""

    def __init__(
        self,
        provider: Optional[str] = None,
        api_key: Optional[str] = None,
        config: Optional[Config] = None,
        **kwargs,
    ):
        """Initialize LLM client.

        Args:
            provider: Provider name (openai, anthropic, google)
            api_key: API key for the provider
            config: Configuration object
            **kwargs: Additional config parameters
        """
        if config is None:
            config_dict = {"provider": provider or "openai"}
            if api_key:
                config_dict["api_key"] = api_key
            config_dict.update(kwargs)
            config = Config(**config_dict)

        self.config = config
        self._provider = self._init_provider()

    # Maps provider name to (module, class_name) for lazy import
    _PROVIDER_MAP = {
        "openai": ("openai", "OpenAIProvider"),
        "anthropic": ("anthropic", "AnthropicProvider"),
        "google": ("google", "GoogleProvider"),
        "azure": ("azure", "AzureOpenAIProvider"),
        "groq": ("groq", "GroqProvider"),
        "mistral": ("mistral", "MistralProvider"),
        "openrouter": ("openrouter", "OpenRouterProvider"),
        "bedrock": ("bedrock", "BedrockProvider"),
        "xai": ("xai", "XAIProvider"),
        "cerebras": ("cerebras", "CerebrasProvider"),
        "cohere": ("cohere", "CohereProvider"),
        "perplexity": ("perplexity", "PerplexityProvider"),
        "deepseek": ("deepseek", "DeepSeekProvider"),
        "together": ("together", "TogetherProvider"),
    }

    def _init_provider(self):
        """Initialize the provider client."""
        entry = self._PROVIDER_MAP.get(self.config.provider)
        if not entry:
            raise ValueError(f"Unknown provider: {self.config.provider}")

        module_name, class_name = entry
        import importlib
        mod = importlib.import_module(f".providers.{module_name}", package="pig_llm")
        provider_class = getattr(mod, class_name)
        return provider_class(self.config)

    def complete(
        self,
        prompt: str,
        system: Optional[str] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion.

        Args:
            prompt: User prompt
            system: Optional system message
            **kwargs: Additional parameters

        Returns:
            Response object with content and metadata
        """
        messages = []
        if system:
            messages.append(Message(role="system", content=system))
        messages.append(Message(role="user", content=prompt))

        return self._provider.complete(
            messages=messages,
            model=kwargs.get("model", self.config.model),
            temperature=kwargs.get("temperature", self.config.temperature),
            max_tokens=kwargs.get("max_tokens", self.config.max_tokens),
            **kwargs,
        )

    def stream(
        self,
        prompt: str,
        system: Optional[str] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion.

        Args:
            prompt: User prompt
            system: Optional system message
            **kwargs: Additional parameters

        Yields:
            StreamChunk objects with content
        """
        messages = []
        if system:
            messages.append(Message(role="system", content=system))
        messages.append(Message(role="user", content=prompt))

        yield from self._provider.stream(
            messages=messages,
            model=kwargs.get("model", self.config.model),
            temperature=kwargs.get("temperature", self.config.temperature),
            max_tokens=kwargs.get("max_tokens", self.config.max_tokens),
            **kwargs,
        )

    def chat(
        self,
        messages: list[Message],
        **kwargs,
    ) -> Response:
        """Generate a chat completion with full message history.

        Args:
            messages: List of Message objects
            **kwargs: Additional parameters (tools, etc.)

        Returns:
            Response object with content and metadata
        """
        model = kwargs.pop("model", self.config.model)
        temperature = kwargs.pop("temperature", self.config.temperature)
        max_tokens = kwargs.pop("max_tokens", self.config.max_tokens)

        return self._provider.complete(
            messages=messages,
            model=model,
            temperature=temperature,
            max_tokens=max_tokens,
            **kwargs,
        )



"""Unified multi-provider LLM API for Python."""

from .client import LLM
from .config import Config
from .models import Message, Response, StreamChunk
from .providers import Provider

__version__ = "0.0.1"

__all__ = [
    "LLM",
    "Config",
    "Message",
    "Response",
    "StreamChunk",
    "Provider",
]



"""Anthropic provider implementation."""

from typing import AsyncIterator, Iterator, Optional

import anthropic

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class AnthropicProvider(Provider):
    """Anthropic (Claude) provider implementation."""

    def __init__(self, config: Config):
        """Initialize Anthropic provider."""
        self.config = config
        self.client = anthropic.Anthropic(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )
        self.async_client = anthropic.AsyncAnthropic(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )

    def _convert_messages(self, messages: list[Message]) -> tuple[Optional[str], list[dict]]:
        """Convert internal messages to Anthropic format.
        
        Returns:
            Tuple of (system_message, messages_list)
        """
        system_message = None
        anthropic_messages = []
        
        for msg in messages:
            if msg.role == "system":
                system_message = msg.content
            else:
                anthropic_messages.append({
                    "role": msg.role,
                    "content": msg.content
                })
        
        return system_message, anthropic_messages

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        system, anthropic_messages = self._convert_messages(messages)
        
        response = self.client.messages.create(
            model=model,
            messages=anthropic_messages,
            system=system,
            temperature=temperature,
            max_tokens=max_tokens or 4096,
            **kwargs,
        )

        # Extract content
        content = ""
        for block in response.content:
            if block.type == "text":
                content += block.text

        usage = {
            "prompt_tokens": response.usage.input_tokens,
            "completion_tokens": response.usage.output_tokens,
            "total_tokens": response.usage.input_tokens + response.usage.output_tokens,
        }

        return Response(
            content=content,
            model=response.model,
            usage=usage,
            finish_reason=response.stop_reason,
            metadata={"id": response.id},
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        system, anthropic_messages = self._convert_messages(messages)
        
        with self.client.messages.stream(
            model=model,
            messages=anthropic_messages,
            system=system,
            temperature=temperature,
            max_tokens=max_tokens or 4096,
            **kwargs,
        ) as stream:
            for text in stream.text_stream:
                yield StreamChunk(content=text, finish_reason=None)

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        system, anthropic_messages = self._convert_messages(messages)
        
        response = await self.async_client.messages.create(
            model=model,
            messages=anthropic_messages,
            system=system,
            temperature=temperature,
            max_tokens=max_tokens or 4096,
            **kwargs,
        )

        content = ""
        for block in response.content:
            if block.type == "text":
                content += block.text

        usage = {
            "prompt_tokens": response.usage.input_tokens,
            "completion_tokens": response.usage.output_tokens,
            "total_tokens": response.usage.input_tokens + response.usage.output_tokens,
        }

        return Response(
            content=content,
            model=response.model,
            usage=usage,
            finish_reason=response.stop_reason,
            metadata={"id": response.id},
        )

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        system, anthropic_messages = self._convert_messages(messages)
        
        async with self.async_client.messages.stream(
            model=model,
            messages=anthropic_messages,
            system=system,
            temperature=temperature,
            max_tokens=max_tokens or 4096,
            **kwargs,
        ) as stream:
            async for text in stream.text_stream:
                yield StreamChunk(content=text, finish_reason=None)



"""Provider implementations."""

from ._base import Provider

# Import providers with graceful fallback for missing dependencies.
# Each provider's SDK is optional — only the ones you use need to be installed.

def _try_import(name, module, attr):
    try:
        mod = __import__(f"pig_llm.providers.{module}", fromlist=[attr])
        return getattr(mod, attr)
    except (ImportError, ModuleNotFoundError):
        return None

OpenAIProvider = _try_import("openai", "openai", "OpenAIProvider")
AnthropicProvider = _try_import("anthropic", "anthropic", "AnthropicProvider")
GoogleProvider = _try_import("google", "google", "GoogleProvider")
AzureOpenAIProvider = _try_import("azure", "azure", "AzureOpenAIProvider")
GroqProvider = _try_import("groq", "groq", "GroqProvider")
MistralProvider = _try_import("mistral", "mistral", "MistralProvider")
OpenRouterProvider = _try_import("openrouter", "openrouter", "OpenRouterProvider")
BedrockProvider = _try_import("bedrock", "bedrock", "BedrockProvider")
XAIProvider = _try_import("xai", "xai", "XAIProvider")
CerebrasProvider = _try_import("cerebras", "cerebras", "CerebrasProvider")
CohereProvider = _try_import("cohere", "cohere", "CohereProvider")
PerplexityProvider = _try_import("perplexity", "perplexity", "PerplexityProvider")
DeepSeekProvider = _try_import("deepseek", "deepseek", "DeepSeekProvider")
TogetherProvider = _try_import("together", "together", "TogetherProvider")

__all__ = [
    "Provider",
    "OpenAIProvider",
    "AnthropicProvider",
    "GoogleProvider",
    "AzureOpenAIProvider",
    "GroqProvider",
    "MistralProvider",
    "OpenRouterProvider",
    "BedrockProvider",
    "XAIProvider",
    "CerebrasProvider",
    "CohereProvider",
    "PerplexityProvider",
    "DeepSeekProvider",
    "TogetherProvider",
]



"""Cohere provider implementation."""

from typing import AsyncIterator, Iterator, Optional

try:
    import cohere
    from cohere import AsyncClient, Client
except ImportError:
    raise ImportError("cohere is required. Install with: pip install cohere")

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class CohereProvider(Provider):
    """Cohere provider implementation (Command models)."""

    def __init__(self, config: Config):
        """Initialize Cohere provider."""
        self.config = config
        
        self.client = Client(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )
        self.async_client = AsyncClient(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )

    def _convert_messages(self, messages: list[Message]) -> tuple[str, str, list[dict]]:
        """Convert internal messages to Cohere format.
        
        Returns:
            Tuple of (preamble/system, final_message, chat_history)
        """
        preamble = ""
        chat_history = []
        final_message = ""
        
        for i, msg in enumerate(messages):
            if msg.role == "system":
                preamble = msg.content
            elif msg.role == "user":
                if i == len(messages) - 1:
                    # Last user message is the prompt
                    final_message = msg.content
                else:
                    chat_history.append({"role": "USER", "message": msg.content})
            elif msg.role == "assistant":
                chat_history.append({"role": "CHATBOT", "message": msg.content})
        
        return preamble, final_message, chat_history

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        preamble, message, chat_history = self._convert_messages(messages)
        
        params = {
            "model": model,
            "message": message,
            "temperature": temperature,
        }
        
        if preamble:
            params["preamble"] = preamble
        if chat_history:
            params["chat_history"] = chat_history
        if max_tokens:
            params["max_tokens"] = max_tokens
        
        response = self.client.chat(**params)
        
        usage = {
            "prompt_tokens": response.meta.tokens.input_tokens if response.meta else 0,
            "completion_tokens": response.meta.tokens.output_tokens if response.meta else 0,
            "total_tokens": (
                response.meta.tokens.input_tokens + response.meta.tokens.output_tokens
                if response.meta
                else 0
            ),
        }
        
        return Response(
            content=response.text,
            model=model,
            usage=usage,
            finish_reason=response.finish_reason,
            metadata={"generation_id": response.generation_id},
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        preamble, message, chat_history = self._convert_messages(messages)
        
        params = {
            "model": model,
            "message": message,
            "temperature": temperature,
        }
        
        if preamble:
            params["preamble"] = preamble
        if chat_history:
            params["chat_history"] = chat_history
        if max_tokens:
            params["max_tokens"] = max_tokens
        
        stream = self.client.chat_stream(**params)
        
        for event in stream:
            if event.event_type == "text-generation":
                yield StreamChunk(
                    content=event.text,
                    finish_reason=None,
                    metadata={},
                )
            elif event.event_type == "stream-end":
                yield StreamChunk(
                    content="",
                    finish_reason=event.finish_reason if hasattr(event, "finish_reason") else "stop",
                    metadata={},
                )

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        preamble, message, chat_history = self._convert_messages(messages)
        
        params = {
            "model": model,
            "message": message,
            "temperature": temperature,
        }
        
        if preamble:
            params["preamble"] = preamble
        if chat_history:
            params["chat_history"] = chat_history
        if max_tokens:
            params["max_tokens"] = max_tokens
        
        response = await self.async_client.chat(**params)
        
        usage = {
            "prompt_tokens": response.meta.tokens.input_tokens if response.meta else 0,
            "completion_tokens": response.meta.tokens.output_tokens if response.meta else 0,
            "total_tokens": (
                response.meta.tokens.input_tokens + response.meta.tokens.output_tokens
                if response.meta
                else 0
            ),
        }
        
        return Response(
            content=response.text,
            model=model,
            usage=usage,
            finish_reason=response.finish_reason,
            metadata={"generation_id": response.generation_id},
        )

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        preamble, message, chat_history = self._convert_messages(messages)
        
        params = {
            "model": model,
            "message": message,
            "temperature": temperature,
        }
        
        if preamble:
            params["preamble"] = preamble
        if chat_history:
            params["chat_history"] = chat_history
        if max_tokens:
            params["max_tokens"] = max_tokens
        
        stream = self.async_client.chat_stream(**params)
        
        async for event in stream:
            if event.event_type == "text-generation":
                yield StreamChunk(
                    content=event.text,
                    finish_reason=None,
                    metadata={},
                )
            elif event.event_type == "stream-end":
                yield StreamChunk(
                    content="",
                    finish_reason=event.finish_reason if hasattr(event, "finish_reason") else "stop",
                    metadata={},
                )



"""Base provider class."""

from abc import ABC, abstractmethod
from typing import AsyncIterator, Iterator, Optional

from ..models import Message, Response, StreamChunk


class Provider(ABC):
    """Base class for LLM providers."""

    @abstractmethod
    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        pass

    @abstractmethod
    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        pass

    @abstractmethod
    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        pass

    @abstractmethod
    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        pass



"""Tests for LLM client."""

import pytest
from unittest.mock import Mock, patch
from pig_llm import LLM, Config, Message


def test_llm_initialization_with_provider():
    """Test LLM initialization with provider."""
    with patch('pig_llm.providers.openai.OpenAIProvider') as MockProvider:
        MockProvider.return_value = Mock()
        llm = LLM(provider="openai", api_key="test-key")
        assert llm.config.provider == "openai"
        assert llm.config.api_key == "test-key"


def test_llm_initialization_with_config():
    """Test LLM initialization with config."""
    config = Config(provider="openai", api_key="test-key", model="gpt-4")
    with patch('pig_llm.providers.openai.OpenAIProvider') as MockProvider:
        MockProvider.return_value = Mock()
        llm = LLM(config=config)
        assert llm.config == config


def test_llm_unknown_provider():
    """Test unknown provider raises error."""
    with pytest.raises((ValueError, Exception)):
        LLM(provider="unknown", api_key="test")


def test_llm_complete_creates_messages():
    """Test complete method creates proper messages."""
    with patch('pig_llm.providers.openai.OpenAIProvider') as MockProvider:
        mock_provider = Mock()
        MockProvider.return_value = mock_provider

        llm = LLM(provider="openai", api_key="test")
        llm.complete("Hello", system="You are helpful")

        assert mock_provider.complete.called
        call_args = mock_provider.complete.call_args
        messages = call_args.kwargs['messages']

        assert len(messages) == 2
        assert messages[0].role == "system"
        assert messages[1].role == "user"


def test_llm_complete_without_system():
    """Test complete without system message."""
    with patch('pig_llm.providers.openai.OpenAIProvider') as MockProvider:
        mock_provider = Mock()
        MockProvider.return_value = mock_provider

        llm = LLM(provider="openai", api_key="test")
        llm.complete("Hello")

        call_args = mock_provider.complete.call_args
        messages = call_args.kwargs['messages']

        assert len(messages) == 1
        assert messages[0].role == "user"


def test_llm_chat():
    """Test chat method with message list."""
    with patch('pig_llm.providers.openai.OpenAIProvider') as MockProvider:
        mock_provider = Mock()
        MockProvider.return_value = mock_provider

        llm = LLM(provider="openai", api_key="test")
        messages = [
            Message(role="user", content="Hello"),
            Message(role="assistant", content="Hi"),
            Message(role="user", content="How are you?"),
        ]
        llm.chat(messages)

        call_args = mock_provider.complete.call_args
        passed_messages = call_args.kwargs['messages']
        assert len(passed_messages) == 3



"""Google Gemini provider implementation."""

import warnings
from typing import AsyncIterator, Iterator, Optional

with warnings.catch_warnings():
    warnings.simplefilter("ignore", FutureWarning)
    import google.generativeai as genai

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class GoogleProvider(Provider):
    """Google Gemini provider implementation."""

    def __init__(self, config: Config):
        """Initialize Google provider."""
        self.config = config
        
        # Configure API
        genai.configure(api_key=config.api_key)
        
        # Create model
        self.model = genai.GenerativeModel(config.model)

    def _convert_messages(self, messages: list[Message]) -> list[dict]:
        """Convert internal messages to Google format.
        
        Returns:
            List of messages in Google format
        """
        # Google uses 'user' and 'model' roles
        google_messages = []
        system_parts = []
        
        for msg in messages:
            if msg.role == "system":
                system_parts.append(msg.content)
            elif msg.role == "assistant":
                google_messages.append({
                    "role": "model",
                    "parts": [msg.content]
                })
            else:  # user
                google_messages.append({
                    "role": "user",
                    "parts": [msg.content]
                })
        
        # Prepend system message as first user message if present
        if system_parts:
            system_content = "\n".join(system_parts)
            if google_messages:
                # Insert at beginning
                google_messages.insert(0, {
                    "role": "user",
                    "parts": [f"System instructions: {system_content}"]
                })
        
        return google_messages

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        google_messages = self._convert_messages(messages)
        
        # Build chat
        chat = self.model.start_chat(history=google_messages[:-1] if len(google_messages) > 1 else [])
        
        # Get last message
        last_message = google_messages[-1]["parts"][0] if google_messages else ""
        
        # Generate response
        response = chat.send_message(
            last_message,
            generation_config=genai.types.GenerationConfig(
                temperature=temperature,
                max_output_tokens=max_tokens,
            )
        )

        # Extract usage if available
        usage = {}
        if hasattr(response, "usage_metadata"):
            usage = {
                "prompt_tokens": response.usage_metadata.prompt_token_count,
                "completion_tokens": response.usage_metadata.candidates_token_count,
                "total_tokens": response.usage_metadata.total_token_count,
            }

        return Response(
            content=response.text,
            model=model,
            usage=usage if usage else None,
            finish_reason=str(response.candidates[0].finish_reason) if response.candidates else None,
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        google_messages = self._convert_messages(messages)
        
        chat = self.model.start_chat(history=google_messages[:-1] if len(google_messages) > 1 else [])
        last_message = google_messages[-1]["parts"][0] if google_messages else ""
        
        response = chat.send_message(
            last_message,
            generation_config=genai.types.GenerationConfig(
                temperature=temperature,
                max_output_tokens=max_tokens,
            ),
            stream=True,
        )

        for chunk in response:
            if chunk.text:
                yield StreamChunk(
                    content=chunk.text,
                    finish_reason=None,
                )

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        # Google SDK doesn't have native async, use sync version
        return self.complete(messages, model, temperature, max_tokens, **kwargs)

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        # Google SDK doesn't have native async, use sync version
        for chunk in self.stream(messages, model, temperature, max_tokens, **kwargs):
            yield chunk



"""Tests for newly added providers."""

import pytest


class TestNewProviders:
    """Test that new providers can be imported and initialized."""

    def test_bedrock_import(self):
        """Test Bedrock provider import."""
        from pig_llm.providers.bedrock import BedrockProvider
        assert BedrockProvider is not None

    def test_xai_import(self):
        """Test xAI provider import."""
        from pig_llm.providers.xai import XAIProvider
        assert XAIProvider is not None

    def test_cerebras_import(self):
        """Test Cerebras provider import."""
        from pig_llm.providers.cerebras import CerebrasProvider
        assert CerebrasProvider is not None

    def test_cohere_import(self):
        """Test Cohere provider import."""
        from pig_llm.providers.cohere import CohereProvider
        assert CohereProvider is not None

    def test_perplexity_import(self):
        """Test Perplexity provider import."""
        from pig_llm.providers.perplexity import PerplexityProvider
        assert PerplexityProvider is not None

    def test_deepseek_import(self):
        """Test DeepSeek provider import."""
        from pig_llm.providers.deepseek import DeepSeekProvider
        assert DeepSeekProvider is not None

    def test_together_import(self):
        """Test Together AI provider import."""
        from pig_llm.providers.together import TogetherProvider
        assert TogetherProvider is not None


class TestProviderRegistration:
    """Test that providers are registered in client."""

    def test_all_providers_in_config(self):
        """Test that all providers are in config literal."""
        from pig_llm.config import Config
        import typing
        
        # Get the Literal type from Config.provider
        provider_field = Config.__fields__['provider']
        # Check that new providers are included
        # This is a basic sanity check
        
    @pytest.mark.parametrize("provider_name", [
        "bedrock",
        "xai",
        "cerebras",
        "cohere",
        "perplexity",
        "deepseek",
        "together",
    ])
    def test_provider_initialization(self, provider_name):
        """Test that each provider can be initialized via LLM client."""
        # This test requires dependencies to be installed
        # Mark as integration test if needed
        pytest.skip("Requires dependencies and API keys - integration test")



"""Data models for LLM interactions."""

from typing import Any, Literal, Optional

from pydantic import BaseModel, Field


class Message(BaseModel):
    """A message in a conversation."""

    role: Literal["system", "user", "assistant", "tool"]
    content: str
    metadata: Optional[dict[str, Any]] = None


class Response(BaseModel):
    """Response from an LLM completion."""

    content: str
    model: str
    usage: Optional[dict[str, int]] = None
    finish_reason: Optional[str] = None
    tool_calls: Optional[list[dict[str, Any]]] = None
    metadata: Optional[dict[str, Any]] = None


class StreamChunk(BaseModel):
    """A chunk from a streaming response."""

    content: str
    finish_reason: Optional[str] = None
    metadata: Optional[dict[str, Any]] = None


class Usage(BaseModel):
    """Token usage information."""

    prompt_tokens: int = 0
    completion_tokens: int = 0
    total_tokens: int = 0

    def __add__(self, other: "Usage") -> "Usage":
        """Add two usage objects together."""
        return Usage(
            prompt_tokens=self.prompt_tokens + other.prompt_tokens,
            completion_tokens=self.completion_tokens + other.completion_tokens,
            total_tokens=self.total_tokens + other.total_tokens,
        )



"""Groq provider implementation (ultra-fast inference)."""

from typing import AsyncIterator, Iterator, Optional

from groq import Groq, AsyncGroq

from ..config import Config
from ..models import Message, Response, StreamChunk
from ._base import Provider


class GroqProvider(Provider):
    """Groq provider implementation (fast LLM inference)."""

    def __init__(self, config: Config):
        """Initialize Groq provider."""
        self.config = config
        self.client = Groq(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )
        self.async_client = AsyncGroq(
            api_key=config.api_key,
            timeout=config.timeout,
            max_retries=config.max_retries,
        )

    def _convert_messages(self, messages: list[Message]) -> list[dict]:
        """Convert internal messages to Groq format."""
        return [{"role": msg.role, "content": msg.content} for msg in messages]

    def complete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Generate a completion."""
        response = self.client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            **kwargs,
        )

        choice = response.choices[0]
        usage = {
            "prompt_tokens": response.usage.prompt_tokens if response.usage else 0,
            "completion_tokens": response.usage.completion_tokens
            if response.usage
            else 0,
            "total_tokens": response.usage.total_tokens if response.usage else 0,
        }

        return Response(
            content=choice.message.content or "",
            model=response.model,
            usage=usage,
            finish_reason=choice.finish_reason,
            metadata={"id": response.id},
        )

    def stream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Iterator[StreamChunk]:
        """Stream a completion."""
        stream = self.client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            stream=True,
            **kwargs,
        )

        for chunk in stream:
            choice = chunk.choices[0]
            if choice.delta.content:
                yield StreamChunk(
                    content=choice.delta.content,
                    finish_reason=choice.finish_reason,
                    metadata={"id": chunk.id},
                )

    async def acomplete(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> Response:
        """Async generate a completion."""
        response = await self.async_client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            **kwargs,
        )

        choice = response.choices[0]
        usage = {
            "prompt_tokens": response.usage.prompt_tokens if response.usage else 0,
            "completion_tokens": response.usage.completion_tokens
            if response.usage
            else 0,
            "total_tokens": response.usage.total_tokens if response.usage else 0,
        }

        return Response(
            content=choice.message.content or "",
            model=response.model,
            usage=usage,
            finish_reason=choice.finish_reason,
            metadata={"id": response.id},
        )

    async def astream(
        self,
        messages: list[Message],
        model: str,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        **kwargs,
    ) -> AsyncIterator[StreamChunk]:
        """Async stream a completion."""
        stream = await self.async_client.chat.completions.create(
            model=model,
            messages=self._convert_messages(messages),
            temperature=temperature,
            max_tokens=max_tokens,
            stream=True,
            **kwargs,
        )

        async for chunk in stream:
            choice = chunk.choices[0]
            if choice.delta.content:
                yield StreamChunk(
                    content=choice.delta.content,
                    finish_reason=choice.finish_reason,
                    metadata={"id": chunk.id},
                )



"""Tests for data models."""

import pytest
from pig_llm.models import Message, Response, StreamChunk, Usage


def test_message_creation():
    """Test message creation."""
    msg = Message(role="user", content="Hello")
    assert msg.role == "user"
    assert msg.content == "Hello"
    assert msg.metadata is None


def test_message_with_metadata():
    """Test message with metadata."""
    msg = Message(
        role="assistant",
        content="Hi",
        metadata={"model": "gpt-4"}
    )
    assert msg.metadata["model"] == "gpt-4"


def test_response_creation():
    """Test response creation."""
    response = Response(
        content="Hello world",
        model="gpt-3.5-turbo",
        usage={"prompt_tokens": 10, "completion_tokens": 5, "total_tokens": 15}
    )
    assert response.content == "Hello world"
    assert response.model == "gpt-3.5-turbo"
    assert response.usage["total_tokens"] == 15


def test_stream_chunk():
    """Test stream chunk."""
    chunk = StreamChunk(content="Hello", finish_reason=None)
    assert chunk.content == "Hello"
    assert chunk.finish_reason is None


def test_usage_addition():
    """Test usage addition."""
    usage1 = Usage(prompt_tokens=10, completion_tokens=5, total_tokens=15)
    usage2 = Usage(prompt_tokens=20, completion_tokens=10, total_tokens=30)
    
    total = usage1 + usage2
    assert total.prompt_tokens == 30
    assert total.completion_tokens == 15
    assert total.total_tokens == 45


def test_invalid_message_role():
    """Test invalid message role."""
    with pytest.raises(ValueError):
        Message(role="invalid", content="test")
